
!UiViewInfo methodsFor: 'private' stamp: 'mt 10/14/2010 13:28:47.076'!
formatHeaderItem! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/14/2010 13:34:19.717'!
headerItem: aMorph
		
	headerItem := aMorph.
	self formatHeaderItem.! !

Morph subclass: #UiItemViewport
	instanceVariableNames: 'rows columns currentCell hoveredCell selectionColor selection selectionMode selectionBehavior verticalHeader horizontalHeader'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 10/13/2010 16:32:59.912'!
addMorph: aMorph atIndex: aModelIndex

	| col row indent |	
	aMorph setProperty: #modelIndex toValue: aModelIndex.
	aMorph setProperty: #isExpanded toValue: false.
	aMorph setProperty: #hasChildren toValue: aModelIndex hasChildren.
	
	row := self addRowFor: aModelIndex.
	row items add: aMorph.
	
	col := self columnFor: aModelIndex.
	col items add: aMorph beforeIndex: (rows indexOf: row).
	
	self addMorphBack: aMorph.
	
	"Correct indent."
	indent := 0.
	(aModelIndex column = 1) ifTrue: [
		(col itemBefore: aMorph)
			ifNil: [indent := self indent] "root"
			ifNotNil: [:item |
				"The item before is my parent. Add additional indentation."
				(item valueOfProperty: #modelIndex) = aModelIndex parent
					ifTrue: [indent := (item valueOfProperty: #indent) + self indent].
				"The item before is my sibling. Use its indentation."
				(item valueOfProperty: #modelIndex) parent = aModelIndex parent
					ifTrue: [indent := item valueOfProperty: #indent]]].
	aMorph setProperty: #indent toValue: indent.! !

!UiItemViewport methodsFor: 'add/remove - private' stamp: 'mt 10/14/2010 10:02:41.166'!
addRowFor: aModelIndex
	"Adds a new row if needed. Returns the correct one if possible."
	
	| row isCorrectRow isPreviousSibling isParentRow |
	isCorrectRow := [:r | r parentModelIndex = aModelIndex parent
		and: [r firstModelIndex row = aModelIndex row]].
	isPreviousSibling := [:r | r parentModelIndex = aModelIndex parent 
		and: [r firstModelIndex row = (aModelIndex row - 1)]].
	isParentRow := [:r | r firstModelIndex = aModelIndex parent].
	
	rows reverseDetect: [:r |
		(isCorrectRow value: r)
			ifTrue: [row := r. true]
			ifFalse: [((isPreviousSibling value: r) or: [isParentRow value: r])
				ifTrue: [row := rows add: UiViewRowInfo new after: r. true]
				ifFalse: [false]]
		] ifNone: [row := rows add: UiViewRowInfo new].
	
	^ row! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 10/11/2010 17:54:55.353'!
cellBoundsFor: aCellIndex

	| col row |
	(aCellIndex x > columns size
		or: [aCellIndex y > rows size])
		ifTrue: [^ 0@0 extent: 0@0].
	
	col := columns at: aCellIndex x.
	row := rows at: aCellIndex y.
	
	^ (self leftForColumn: col) @ (self topForRow: row)
		extent: col width @ row height! !

!UiItemViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 10/9/2010 12:16:36.293'!
cellIndexFor: aModelIndex

	| rowIndex col |
	col := self columnFor: aModelIndex.
	rowIndex := col itemIndexFor: aModelIndex.

	^ aModelIndex column @ rowIndex! !

!UiItemViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 10/9/2010 12:58:33.455'!
cellIndexFromPoint: aPoint
	"Given a point in pixel coordinates."
	
	| offset |
	offset := self topLeft.
	columns withIndexDo: [:colInfo :colIndex |
		rows withIndexDo: [:rowInfo :rowIndex | 
			((offset extent: colInfo width @ rowInfo height) containsPoint: aPoint)
				ifTrue: [^ colIndex @ rowIndex].
			offset := offset translateBy: 0 @ rowInfo height].
		offset := (offset x + colInfo width) @ self top].
	
	^ nil! !

!UiItemViewport methodsFor: 'initialization' stamp: 'mt 10/11/2010 17:56:54.347'!
clear

	self currentCell: nil.
	self selection clear.
	
	columns do: [:colInfo | colInfo clear].
	rows removeAll.
	self removeAllMorphs.! !

!UiItemViewport methodsFor: 'signals' stamp: 'mt 10/8/2010 16:05:54.303'!
clicked: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 10/14/2010 09:45:01.756'!
collapseRow: aRowInfo

	| foundFirst predecessors |
	foundFirst := false.
	predecessors := {aRowInfo} asOrderedCollection.
	rows
		select: [:row |
			"Find all rows and subrows that need to be removed."
			(predecessors anySatisfy: [:pred | row parentModelIndex = pred firstModelIndex])
				ifTrue: [predecessors add: row. foundFirst := true]
				ifFalse: [foundFirst ifTrue: [predecessors := #()]. false]]
		thenDo: [:row | self removeRow: row].! !

!UiItemViewport methodsFor: 'rows/columns - to info' stamp: 'mt 10/11/2010 16:18:23.881'!
columnAt: index

	^ columns
		at: index
		ifAbsentPut: [UiViewColumnInfo new]! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 10/11/2010 17:58:53.347'!
columnBoundsFor: aColIndex

	| col |
	aColIndex > columns size
		ifTrue: [^ 0@0 extent: 0@0].
		
	col := columns at: aColIndex.
	
	^ (self leftForColumn: col) @ self top
		extent: col width @ self height! !

!UiItemViewport methodsFor: 'rows/columns - to info' stamp: 'mt 10/9/2010 11:26:53.355'!
columnFor: aModelIndex

	^ columns
		at: aModelIndex column
		ifAbsentPut: [UiViewColumnInfo new]! !

!UiItemViewport methodsFor: 'iterating' stamp: 'mt 10/14/2010 10:25:48.766'!
columnsDo: aBlock

	columns do: aBlock.! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/11/2010 14:08:10.466'!
currentCell

	^ currentCell! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/11/2010 17:24:32.458'!
currentCell: aPoint

	aPoint = currentCell
		ifTrue: [^ self].
		
	currentCell := aPoint.
	self redrawLater.
	
	currentCell ifNotNil: [
		self focusChanged: (self cellBoundsFor: currentCell)].! !

!UiItemViewport methodsFor: 'event handling' stamp: 'mt 10/9/2010 13:06:27.424'!
doubleClick: evt

	self doubleClicked: (self modelIndexFor: (self cellIndexFromPoint: evt position)).! !

!UiItemViewport methodsFor: 'signals' stamp: 'mt 10/8/2010 16:05:00.799'!
doubleClicked: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 14:27:00.022'!
drawCurrentCellOn: aCanvas

	self currentCell ifNil: [^ self].
	
	self selectionBehavior = #selectCells ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self cellBoundsFor: self currentCell)
			color: self selectionColor darker.
		^ self].
		
	self selectionBehavior = #selectRows ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self rowBoundsFor: self currentCell y)
			color: self selectionColor darker.
		^ self].
	
	self selectionBehavior = #selectColumns ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self columnBoundsFor: self currentCell x)
			color: self selectionColor darker.
		^ self]! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 14:34:39.295'!
drawHighlightOn: aCanvas bounds: drawBounds color: aColor
	
	| gradient |

	gradient := (GradientFillStyle ramp: {
			0.0 -> Color white.
			0.1-> (aColor adjustBrightness: 0.05).
			0.6 -> (aColor darker)})
		origin: drawBounds topLeft;
		direction: 0 @ drawBounds height.
		
	aCanvas
		fillRoundRect: (drawBounds insetBy: (1@1 corner: 0@0))
		radius: 3
		fillStyle: gradient;
		frameRoundRect: (drawBounds outsetBy: (0@0 corner: 1@1))
		radius: 3
		width: 1
		color: aColor muchDarker.! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 14:41:13.054'!
drawHoveredCellOn: aCanvas

	self hoveredCell ifNil: [^ self].
	
	self selectionBehavior = #selectCells ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self cellBoundsFor: self hoveredCell)
			color: self selectionColor muchLighter.
		^ self].
		
	self selectionBehavior = #selectRows ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self rowBoundsFor: self hoveredCell y)
			color: self selectionColor muchLighter.
		^ self].
	
	self selectionBehavior = #selectColumns ifTrue: [
		self
			drawHighlightOn: aCanvas
			bounds: (self columnBoundsFor: self hoveredCell x)
			color: self selectionColor muchLighter.
		^ self]! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 10/14/2010 13:16:54.342'!
drawOn: aCanvas
	
	self
		drawHoveredCellOn: aCanvas;
		drawSelectionOn: aCanvas;
		drawCurrentCellOn: aCanvas.
	super drawOn: aCanvas.! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 09:25:33.057'!
drawSelectionOn: aCanvas

	self selectionBehavior = #selectCells ifTrue: [
		self selection cells do: [:cell |
			self
				drawSelectionOn: aCanvas
				bounds: (self cellBoundsFor: cell)	].
			^ self].
		
	self selectionBehavior = #selectRows ifTrue: [
		(self selection cells collect: [:cell | cell y]) asSet do: [:rowIndex |
			self
				drawSelectionOn: aCanvas
				bounds: (self rowBoundsFor: rowIndex)].
		^ self].
	
	self selectionBehavior = #selectColumns ifTrue: [
		(self selection cells collect: [:cell | cell x]) asSet do: [:colIndex |
			self
				drawSelectionOn: aCanvas
				bounds: (self columnBoundsFor: colIndex)].
		^ self]! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 14:17:16.138'!
drawSelectionOn: aCanvas bounds: drawBounds
	
	self
		drawHighlightOn: aCanvas
		bounds: drawBounds
		color: self selectionColor.! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 10/11/2010 16:14:39.006'!
drawSubmorphsOn: aCanvas 
	"Clip in cells. Optimized."
	
	| offset |
	submorphs isEmpty ifTrue: [^self].
	
	offset := self topLeft.
	columns withIndexDo: [:colInfo :colIndex |
		
		colInfo items withIndexDo: [:item :rowIndex | | rowInfo |
			rowInfo := rows at: rowIndex.
			aCanvas
				clipBy: (offset extent: colInfo width @ rowInfo height)
				during: [:canvas | canvas fullDrawMorph: item].
			
			offset := offset translateBy: 0 @ rowInfo height].
		
		offset := (offset x + colInfo width) @ self top].! !

!UiItemViewport methodsFor: 'geometry' stamp: 'mt 10/10/2010 13:03:00.575'!
extent: aPoint

	super extent: aPoint.
	self resizedToWidth: aPoint x height: aPoint y.! !

!UiItemViewport methodsFor: 'signals' stamp: 'mt 10/11/2010 17:18:00.533'!
focusChanged: aRectangle
	"Indicate that a scrollable container should ensure the given area as visible."
	
	self emit.! !

!UiItemViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 14:52:42.129'!
handleMouseMove: anEvent

	super handleMouseMove: anEvent.
	
	anEvent wasHandled not
	"	and: [anEvent hand mouseFocus == self])"
		ifTrue: [self mouseMove: anEvent. anEvent wasHandled: true]! !

!UiItemViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 16:53:44.92'!
handlesKeyboard: evt

	^ true! !

!UiItemViewport methodsFor: 'event handling' stamp: 'mt 10/8/2010 15:30:36.577'!
handlesMouseDown: evt

	^ true! !

!UiItemViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 14:46:59'!
handlesMouseOver: evt

	^ true! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:00:05.566'!
horizontalHeader

	^ horizontalHeader! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:00:11.666'!
horizontalHeader: aMorph

	horizontalHeader := aMorph.! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/11/2010 14:39:13.856'!
hoveredCell

	^ hoveredCell! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/11/2010 17:13:05.489'!
hoveredCell: aPoint

	aPoint = hoveredCell
		ifTrue: [^ self].
		
	hoveredCell := aPoint.
	self redrawLater.! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/14/2010 13:14:56.09'!
indent

	^ 0! !

!UiItemViewport methodsFor: 'initialization' stamp: 'mt 10/11/2010 09:04:54.243'!
initialize

	super initialize.
	self color: Color transparent.
		
	rows := OrderedCollection new.
	columns := OrderedCollection new.
	
	self selectionMode: UiViewSingleSelection.! !

!UiItemViewport methodsFor: 'testing' stamp: 'mt 10/14/2010 10:50:19.462'!
isExpanded: aModelIndex

	^ aModelIndex isNil "the root is always expanded"
		or: [(self rowFor: aModelIndex) isExpanded]! !

!UiItemViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 18:00:37.316'!
keyStroke: evt

	"Manuall navigation."
	self currentCell ifNil: [^ self currentCell: 1@1].

	evt keyCharacter = Character arrowUp
		ifTrue: [self currentCell: (self currentCell - (0@1) max: 1@1)].
		
	evt keyCharacter = Character arrowDown
		ifTrue: [self currentCell: (self currentCell + (0@1) min: (columns size @ rows size) max: 1@1)].
		
	evt keyCharacter = Character arrowLeft
		ifTrue: [self currentCell: (self currentCell - (1@0) max: 1@1)].
		
	evt keyCharacter = Character arrowRight
		ifTrue: [self currentCell: (self currentCell + (1@0) min: (columns size @ rows size) max: 1@1)].
		
	self selection
		add: self currentCell
		event: evt.! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 10/9/2010 13:12:00.791'!
leftForColumn: aColumnInfo

	| result |
	result := self left.
	columns do: [:col |
		col = aColumnInfo ifTrue: [^ result].
		result := result + col width].
	
	Error signal: 'Column info not found!!'.! !

!UiItemViewport methodsFor: 'rows/columns - to model index' stamp: 'mt 10/9/2010 17:25:18.053'!
modelIndexFor: aCellIndex

	aCellIndex ifNil: [^ nil].
	
	^ ((columns at: aCellIndex x) items at: aCellIndex y)
		valueOfProperty: #modelIndex! !

!UiItemViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 17:05:42.001'!
mouseDown: evt

	self currentCell: (self cellIndexFromPoint: evt position).
	self selection
		add: self currentCell
		event: evt.
		
	evt hand
		waitForClicksOrDrag: self
		event: evt
		selectors: {#click:. #doubleClick:. nil. #startDrag:}
		threshold: 1. ! !

!UiItemViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 16:57:04.191'!
mouseEnter: evt

	evt hand newKeyboardFocus: self.! !

!UiItemViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 14:49:23.957'!
mouseLeave: evt

	self hoveredCell: nil.! !

!UiItemViewport methodsFor: 'event handling' stamp: 'mt 10/11/2010 14:49:27.815'!
mouseMove: evt

	self hoveredCell: (self cellIndexFromPoint: evt position).! !

!UiItemViewport methodsFor: 'add/remove - private' stamp: 'mt 10/14/2010 11:37:04.05'!
removeRow: aRowInfo

	aRowInfo items withIndexDo: [:item :index |
		(columns at: index) items remove: item].
	
	self removeAllMorphsIn: aRowInfo items.
	rows remove: aRowInfo.! !

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 10/14/2010 11:40:59.344'!
removeRowFor: aParentModelIndex at: rowIndex

	self removeRow: (rows detect: [:row |
		row parentModelIndex = aParentModelIndex and: [row firstModelIndex row = rowIndex]])! !

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 10/14/2010 10:45:00.46'!
replaceMorphWith: aMorph atIndex: aModelIndex

	| oldMorph col row indent |
	oldMorph := (rows detect: [:rowInfo | rowInfo includes: aModelIndex]) itemAt: aModelIndex.
	
	aMorph setProperty: #modelIndex toValue: aModelIndex.
	aMorph setProperty: #isExpanded toValue: (oldMorph valueOfProperty: #isExpanded).
	aMorph setProperty: #hasChildren toValue: aModelIndex hasChildren.
	
	row := self rowFor: aModelIndex.
	row items
		add: aMorph after: oldMorph;
		remove: oldMorph.
	
	col := self columnFor: aModelIndex.
	col items
		add: aMorph after: oldMorph;
		remove: oldMorph.
	
	self
		removeMorph: oldMorph;
		addMorphBack: aMorph.
	
	"Correct indent."
	indent := 0.
	(aModelIndex column = 1) ifTrue: [
		(col itemBefore: aMorph)
			ifNil: [indent := self indent] "root"
			ifNotNil: [:item |
				"The item before is my parent. Add additional indentation."
				(item valueOfProperty: #modelIndex) = aModelIndex parent
					ifTrue: [indent := (item valueOfProperty: #indent) + self indent].
				"The item before is my sibling. Use its indentation."
				(item valueOfProperty: #modelIndex) parent = aModelIndex parent
					ifTrue: [indent := item valueOfProperty: #indent]]].
	aMorph setProperty: #indent toValue: indent.! !

!UiItemViewport methodsFor: 'signals' stamp: 'mt 10/10/2010 13:02:41.787'!
resizedToWidth: newWidth height: newHeight

	self emit.! !

!UiItemViewport methodsFor: 'rows/columns - to info' stamp: 'mt 10/10/2010 15:08:43.412'!
rowAt: index

	^ rows at: index! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 10/11/2010 17:58:29.235'!
rowBoundsFor: aRowIndex

	| row |
	aRowIndex > rows size
		ifTrue: [^ 0@0 extent: 0@0].
		
	row := rows at: aRowIndex.
	
	^ self left @ (self topForRow: row)
		extent: self width @ row height! !

!UiItemViewport methodsFor: 'rows/columns - to info' stamp: 'mt 10/9/2010 16:12:41.64'!
rowFor: aModelIndex

	^ rows detect: [:row | row includes: aModelIndex]! !

!UiItemViewport methodsFor: 'iterating' stamp: 'mt 10/14/2010 10:25:55.249'!
rowsDo: aBlock

	rows do: aBlock.! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/10/2010 20:03:34.835'!
selection

	^ selection! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 09:12:39.162'!
selection: aSelection

	selection := aSelection.
	self connect: aSelection signal: #selectionChanged toSelector: #redrawLater.
	
	self redrawLater.! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/10/2010 20:13:12.402'!
selectionBehavior

	^ selectionBehavior ifNil: [selectionBehavior := #selectRows]! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 09:28:20.274'!
selectionBehavior: aSymbol
	"#selectCells, #selectRows, #selectColumns"

	selectionBehavior := aSymbol.
	self redrawLater.! !

!UiItemViewport methodsFor: 'signals' stamp: 'mt 10/8/2010 16:07:34.586'!
selectionChanged: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 14:30:17.153'!
selectionColor

	^ selectionColor ifNil: [selectionColor := Color r: 0.772 g: 0.858 b: 1.0]! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/8/2010 14:37:57.041'!
selectionColor: aColor

	selectionColor := aColor.! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 09:04:47.949'!
selectionMode

	^ selectionMode! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 09:04:07.606'!
selectionMode: aSelectionClass

	selectionMode := aSelectionClass.
	self selection: aSelectionClass new.! !

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 13:59:08.486'!
setHeaderItem: aMorph atColumn: anInteger

	(columns at: anInteger) headerItem: aMorph.! !

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 10/12/2010 11:50:38.889'!
setHeaderItem: aMorph atRow: anInteger

	(rows at: anInteger) headerItem: aMorph.! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 10/9/2010 13:11:55.553'!
topForRow: aRowInfo

	| result |
	result := self top.
	rows do: [:row |
		row = aRowInfo ifTrue: [^ result].
		result := result + row height].
	
	Error signal: 'Row info not found!!'.! !

!UiItemViewport methodsFor: 'geometry' stamp: 'mt 10/11/2010 18:07:00.708'!
updateExtent

	self extent:
		(columns
			ifEmpty: [0]
			ifNotEmpty: [(columns collect: [:col | col width]) sum])
		@
		(rows
			ifEmpty: [0]
			ifNotEmpty: [(rows collect: [:row | row height]) sum])! !

!UiItemViewport methodsFor: 'layouting' stamp: 'mt 10/14/2010 13:07:53.401'!
updateLayout

	| heights widths |
	columns do: [:column | column updateSize].
	rows do: [:row | row updateSize].

	heights := rows collect: [:r | r height].
	widths := columns collect: [:c | c width].
	
	columns do: [:column | column layoutItemsWithHeights: heights].
	rows do: [:row | row layoutItemsWithWidths: widths].
	
	columns first items do: [:item |
		item left: self left + (item valueOfProperty: #indent)].

	"Relayout changed header items."
	self horizontalHeader ifNotNil: [:hh |
		hh removeAllMorphs.
		columns do: [:colInfo | hh addMorphBack: colInfo headerItem]].
	self verticalHeader ifNotNil: [:vh |
		vh removeAllMorphs.
		rows do: [:rowInfo | vh addMorphBack: rowInfo headerItem]].
	
	"Shrink wrap to contents."
	self updateExtent.! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 13:59:56.367'!
verticalHeader

	^ verticalHeader! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 13:59:50.126'!
verticalHeader: aMorph

	verticalHeader := aMorph.! !

UiItemViewport subclass: #UiListViewport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

UiItemViewport subclass: #UiTreeViewport
	instanceVariableNames: 'indent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 10/14/2010 13:17:52.299'!
click: evt

	| cell row item |
	cell := self cellIndexFromPoint: evt position.
	cell ifNil: [^ self].
	
	row := rows at: cell y.
	item := row items first.
	
	"Recognize a click on the decoration."
	(evt position x < item left
		and: [evt position x >= (item left - self indent)])
		ifTrue: [self decorationClicked: (item valueOfProperty: #modelIndex)].! !

!UiTreeViewport methodsFor: 'signals' stamp: 'mt 10/14/2010 13:13:30.709'!
decorationClicked: aModelIndex
	"Whenever the user clicks the expand/collapse decoration in front of an item."
	
	self emit.! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 10/14/2010 13:16:22.359'!
drawOn: aCanvas
	
	super drawOn: aCanvas.
	self drawTreeStructureOn: aCanvas.! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 10/14/2010 13:16:36.739'!
drawTreeStructureOn: aCanvas
	
	| first last |
	columns ifEmpty: [^ self].
	
	columns first width < aCanvas clipRect left
		ifTrue: [^ self].
	
	first := self cellIndexFromPoint: (self bounds pointNearestTo: aCanvas clipRect topLeft).
	last := self cellIndexFromPoint: (self bounds pointNearestTo: aCanvas clipRect bottomLeft).

	first ifNil: [first := 1@1].
	last ifNil: [last := 1@rows size].
	
	last < first ifTrue: [^ self].

	first y to: last y do: [:rowIndex |
		| item cellBounds form |
		cellBounds := self cellBoundsFor: 1@rowIndex.
		item := (rows at: rowIndex) items first.
		form := UiSilkIcons bulletArrowDownIcon.
		form := (item valueOfProperty: #hasChildren)
			ifFalse: [nil]
			ifTrue: [(item valueOfProperty: #isExpanded)
				ifFalse: [form rotateBy: #left centerAt: 8@8]
				ifTrue: [form]].

		form ifNotNil: [
			aCanvas
				translucentImage: form
				at: (cellBounds left + ((item valueOfProperty: #indent) - self indent))
					@ (cellBounds top + (cellBounds height - form height // 2))]].! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/14/2010 13:14:25.565'!
indent

	^ indent ifNil: [indent := 12]! !

ScrollPane subclass: #UiAbstractItemView
	instanceVariableNames: 'itemDelegates verticalHeader horizontalHeader vHeaderScroller hHeaderScroller'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiAbstractItemView methodsFor: 'model updates' stamp: 'mt 10/14/2010 13:09:27.159'!
changeColumnsFor: aParentModelIndex from: start to: end

	| index item |
	
	(self viewport isExpanded: aParentModelIndex)
		ifFalse: [^ self].
	
	1 to: (self model rowCountFor: aParentModelIndex) do: [:row |
		start to: end do: [:col |
			index := self model indexRow: row column: col parent: aParentModelIndex.
			item := (self itemDelegateFor: index) itemFor: index.
			self viewport replaceMorphWith: item atIndex: index.
			item fadeIn]].
	
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'model updates' stamp: 'mt 10/14/2010 13:09:37.343'!
changeRowsFor: aParentModelIndex from: start to: end

	| index item |
	
	(self viewport isExpanded: aParentModelIndex)
		ifFalse: [^ self].
	
	start to: end do: [:row |
		1 to: (self model columnCountFor: aParentModelIndex) do: [:col |
			index := self model indexRow: row column: col parent: aParentModelIndex.
			item := (self itemDelegateFor: index) itemFor: index.
			self viewport replaceMorphWith: item atIndex: index.
			item fadeIn]].
	
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'item delegation' stamp: 'mt 10/9/2010 10:54:44.107'!
defaultItemDelegate

	^ itemDelegates at: #default! !

!UiAbstractItemView methodsFor: 'model access' stamp: 'mt 10/11/2010 18:05:14.956'!
fillViewportFor: aModelIndex animated: aBoolean

	| index item |
	self model ifNil: [^ self].
	
	1 to: (self model rowCountFor: aModelIndex) do: [:row |
		1 to: (self model columnCountFor: aModelIndex) do: [:col |
			index := self model indexRow: row column: col parent: aModelIndex.
			item := (self itemDelegateFor: index) itemFor: index.
			self viewport addMorph: item atIndex: index.
			aBoolean ifTrue: [item fadeIn]]].! !

!UiAbstractItemView methodsFor: 'geometry' stamp: 'mt 10/10/2010 15:13:49.001'!
hLeftoverScrollRange

	scroller hasSubmorphs ifFalse:[^0].
	^ (self viewport width - scroller width + 5) max: 0! !

!UiAbstractItemView methodsFor: 'scrolling' stamp: 'mt 10/10/2010 13:46:48.63'!
hScrollBarValue: scrollValue

	super hScrollBarValue: scrollValue.
	hHeaderScroller offset: scroller offset x @ 0.! !

!UiAbstractItemView methodsFor: 'testing' stamp: 'mt 10/14/2010 13:04:29.063'!
hasHorizontalHeader

	^ hHeaderScroller owner notNil! !

!UiAbstractItemView methodsFor: 'testing' stamp: 'mt 10/14/2010 13:03:13.199'!
hasVerticalHeader

	^ vHeaderScroller owner notNil! !

!UiAbstractItemView methodsFor: 'initialization' stamp: 'mt 10/14/2010 12:10:54.535'!
initialize

	"Initialize headers."
	verticalHeader := Morph new color: Color transparent; width: self scrollBarThickness; changeTableLayout; listDirection: #topToBottom.
	horizontalHeader := Morph new color: Color transparent; height: self scrollBarThickness; changeTableLayout; listDirection: #leftToRight.
	vHeaderScroller := TransformMorph new color: Color transparent.
	hHeaderScroller := TransformMorph new color: Color transparent.

	super initialize.
	
	self
		borderWidth: 0;
		color: Color white.
	
	"Initialize viewport."
	self scroller addMorph: self viewportClass new.
	self connect: self viewport signal: #focusChanged: toSelector: #scrollToShow:.
	self viewport verticalHeader: verticalHeader.
	self viewport horizontalHeader: horizontalHeader.
	
	"Initialize item delegates."
	itemDelegates := Dictionary new.
	itemDelegates at: #default put: UiSimpleItemDelegate new.
	
	"Initialize headers - Finish."
	verticalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #height: pattern: #(2).
	vHeaderScroller addMorph: verticalHeader.

	horizontalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #width:.
	hHeaderScroller addMorph: horizontalHeader.! !

!UiAbstractItemView methodsFor: 'geometry' stamp: 'mt 10/14/2010 13:04:32.651'!
innerBounds

	| inner |
	inner := super innerBounds.
	
	self hasVerticalHeader ifTrue: [
		inner := inner insetBy: (verticalHeader width @ 0 corner: 0@0)].

	self hasHorizontalHeader ifTrue: [
		inner := inner insetBy: (0 @ horizontalHeader height corner: 0@0)].

	^ inner! !

!UiAbstractItemView methodsFor: 'model updates' stamp: 'mt 10/14/2010 13:09:44.283'!
insertRowsFor: aParentModelIndex from: start to: end

	| index item |
	
	(self viewport isExpanded: aParentModelIndex)
		ifFalse: [^ self].
	
	start to: end do: [:row |
		1 to: (self model columnCountFor: aParentModelIndex) do: [:col |
			index := self model indexRow: row column: col parent: aParentModelIndex.
			item := (self itemDelegateFor: index) itemFor: index.
			self viewport addMorph: item atIndex: index.
			item fadeIn]].
	
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'item delegation' stamp: 'mt 10/10/2010 10:34:00.872'!
itemDelegateFor: aModelIndex

	^ self defaultItemDelegate! !

!UiAbstractItemView methodsFor: 'initialization' stamp: 'mt 10/14/2010 11:36:35.075'!
model: aModel

	self model ifNotNil: [:m | m disconnectAllSignals].
	
	super model: aModel.
	self reset.
	
	self model ifNotNil: [:m |
		self connect: m signal: #modelReset toSelector: #reset.
		self connect: m signal: #rowsInsertedFor:from:to: toSelector: #insertRowsFor:from:to:.
		self connect: m signal: #rowsChangedFor:from:to: toSelector: #changeRowsFor:from:to:.
		self connect: m signal: #rowsRemovedFor:from:to: toSelector: #removeRowsFor:from:to:.].! !

!UiAbstractItemView methodsFor: 'model updates' stamp: 'mt 10/14/2010 13:09:51.695'!
removeRowsFor: aParentModelIndex from: start to: end

	(self viewport isExpanded: aParentModelIndex)
		ifFalse: [^ self].
	
	start to: end do: [:row |
		self viewport removeRowFor: aParentModelIndex at: row].
	
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'model access' stamp: 'mt 10/14/2010 13:09:59.44'!
reset
	self viewport clear.
	
	self fillViewportFor: nil animated: false.
	self resetHeaderItems.
	
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'model access' stamp: 'mt 10/14/2010 13:06:56.299'!
resetHeaderItems

	| index item |
	self model ifNil: [^ self].
	
	self hasVerticalHeader ifTrue: [
		1 to: (self model rowCountFor: nil) do: [:row |
			index := self model indexRow: row column: 0 parent: nil.
			item := (self itemDelegateFor: index) itemFor: index.
			verticalHeader width: (verticalHeader width max: item width).
			self viewport setHeaderItem: item atRow: row]].
	
	self hasHorizontalHeader ifTrue: [
		1 to: (self model columnCountFor: nil) do: [:col |
			index := self model indexRow: 0 column: col parent: nil.
			item := (self itemDelegateFor: index) itemFor: index.
			horizontalHeader height: (horizontalHeader height max: item height).
			self viewport setHeaderItem: item atColumn: col]].! !

!UiAbstractItemView methodsFor: 'geometry' stamp: 'mt 10/10/2010 14:42:58.971'!
resizeScroller

	super resizeScroller.
	
	hHeaderScroller bounds: (scroller left @ self top
		extent: (scroller width @ horizontalHeader height)).
	vHeaderScroller bounds: (self left @ scroller top
		extent: (verticalHeader width @ scroller height)).! !

!UiAbstractItemView methodsFor: 'geometry' stamp: 'mt 10/10/2010 13:11:49.345'!
scrollBarThickness
	"Includes border"

	^ Preferences scrollBarsNarrow
			ifTrue: [10]
			ifFalse: [14].! !

!UiAbstractItemView methodsFor: 'scrolling' stamp: 'mt 10/11/2010 17:35:05.273'!
scrollToShow: aRectangle
	"Scroll to include as much of aRectangle as possible, where aRectangle is in the scroller's local space."
	
	|offset|
	offset := self scroller offset.
	
	(aRectangle right - offset x) > self innerBounds width
		ifTrue: [offset := (aRectangle left - self innerBounds width + aRectangle width min: self viewport width - self innerBounds width) @ offset y].
	(aRectangle left - offset x) < 0
		ifTrue: [offset := (aRectangle left min: self viewport width - self innerBounds width) @ offset y].
	
	
	(aRectangle top - offset y) < 0
		ifTrue: [offset := offset x @ (
					(aRectangle top min: self viewport height - self innerBounds height))].
	(aRectangle bottom - offset y) > self innerBounds height
		ifTrue: [offset := offset x @ (
					(aRectangle top - self innerBounds height + aRectangle height min: self viewport height - self innerBounds height))].
		
	offset = self scroller offset ifFalse: [
		self scroller offset: offset.
		self setScrollDeltas]! !

!UiAbstractItemView methodsFor: 'rows/columns - resizing' stamp: 'mt 10/14/2010 13:10:06.532'!
setHeightForRow: anIndex to: anInteger

	| row |
	row := self viewport rowAt: anIndex.
	row resizeMode ~= #rigid
		ifTrue: [^ self].
		
	row height: anInteger.
		
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - resizing' stamp: 'mt 10/14/2010 13:10:12.957'!
setResizeModeForColumn: anIndex to: aSymbol

	(self viewport columnAt: anIndex)
		resizeMode: aSymbol.
		
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - resizing' stamp: 'mt 10/14/2010 13:10:24.149'!
setResizeModeForRow: anIndex to: aSymbol

	(self viewport rowAt: anIndex)
		resizeMode: aSymbol.
		
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - resizing' stamp: 'mt 10/14/2010 13:10:34.713'!
setWidthForColumn: anIndex to: anInteger

	| col |
	col := self viewport columnAt: anIndex.
	col resizeMode ~= #rigid
		ifTrue: [^ self].
		
	col width: anInteger.
		
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'layouting' stamp: 'mt 10/14/2010 13:09:15.696'!
updateLayout

	self viewport updateLayout.
	self updateScrollBars.! !

!UiAbstractItemView methodsFor: 'scrolling' stamp: 'mt 10/10/2010 12:03:46.492'!
updateScrollBars

	self
		resizeScrollBars;
		resizeScroller;
		hideOrShowScrollBars;
		hSetScrollDelta;
		vSetScrollDelta.! !

!UiAbstractItemView methodsFor: 'geometry' stamp: 'mt 10/10/2010 15:10:56.555'!
vLeftoverScrollRange

	scroller hasSubmorphs ifFalse:[^0].
	^ (self viewport height - scroller height + 5) max: 0! !

!UiAbstractItemView methodsFor: 'scrolling' stamp: 'mt 10/10/2010 13:46:57.085'!
vScrollBarValue: scrollValue

	super vScrollBarValue: scrollValue.
	vHeaderScroller offset: 0 @ scroller offset y.! !

!UiAbstractItemView methodsFor: 'accessing - private' stamp: 'mt 10/7/2010 17:39:54.815'!
viewport
	"Return the morph that is scrolled."
	
	^ self scroller submorphs first! !

!UiAbstractItemView methodsFor: 'accessing - private' stamp: 'mt 10/14/2010 12:08:54.27'!
viewportClass

	self subclassResponsibility.! !

UiAbstractItemView subclass: #UiListView2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiListView2 methodsFor: 'as yet unclassified' stamp: ''!
viewportClass
	^ UiListViewport! !

UiAbstractItemView subclass: #UiTableView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTableView methodsFor: 'initialization' stamp: 'mt 10/14/2010 12:12:23.38'!
initialize

	super initialize.
		
	self
		addMorph: vHeaderScroller;
		addMorph: hHeaderScroller.
	
	self resizeScroller.! !

!UiTableView methodsFor: 'accessing - private' stamp: 'mt 10/14/2010 13:39:14.187'!
viewportClass

	^ UiItemViewport! !

UiAbstractItemView subclass: #UiTreeView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTreeView methodsFor: 'initialization' stamp: 'mt 10/14/2010 12:11:00.927'!
initialize

	super initialize.
	
	self connect: self viewport signal: #doubleClicked: toSelector: #toggleIndex:.
	self connect: self viewport signal: #decorationClicked: toSelector: #toggleIndex:.
	
	self
		"addMorph: vHeaderScroller;"
		addMorph: hHeaderScroller.
	
	self resizeScroller.! !

!UiTreeView methodsFor: 'model access' stamp: 'mt 10/14/2010 13:10:43.16'!
toggleIndex: aModelIndex

	| row |
	
	row := self viewport rowFor: aModelIndex.
	
	row isExpanded
		ifTrue: [
			row items first setProperty: #isExpanded toValue: false.
			self viewport collapseRow: row]
		ifFalse: [
			row items first setProperty: #isExpanded toValue: true.
			self
				fillViewportFor: row firstModelIndex
				animated: true].
	
	self updateLayout.! !

!UiTreeView methodsFor: 'accessing - private' stamp: 'mt 10/14/2010 13:21:16.162'!
viewportClass

	^ UiTreeViewport! !

!UiViewRowInfo methodsFor: 'private' stamp: 'mt 10/14/2010 13:37:25.357'!
formatHeaderItem

	| backgroundColor |
	backgroundColor := Color gray: 0.85.
	headerItem
		hResizing: #spaceFill;
		fillStyle: ((GradientFillStyle ramp: {
			0.0 -> Color white.
			0.1-> (backgroundColor adjustBrightness: 0.05).
			0.6 -> (backgroundColor darker)})
			origin: headerItem topLeft;
			direction: headerItem width @ 0);
		borderStyle: (BorderStyle raised width: 1).! !

!UiSmalltalkModel methodsFor: 'notifications' stamp: 'mt 10/14/2010 11:43:56.986'!
processNotification: anEvent

	| pkg pkgIndex sysIndex classIndex |
	(anEvent itemKind ~= #category and: [anEvent itemKind ~= #class])
		ifTrue: [^ self].
	
	pkg := PackageOrganizer default
		packageOfSystemCategory: anEvent itemCategory
		ifNone: [^ self].
	pkgIndex := (UiSimpleModelIndex
		row: (PackageOrganizer default packages indexOf: pkg)
		column: 1)
		model: self;
		setProperty: #package toValue: pkg.
	sysIndex := self
		indexRow: (pkg systemCategories indexOf: anEvent itemCategory)
		column: 1
		parent: pkgIndex.
	classIndex := self
		indexRow: ((SystemOrganization listAtCategoryNamed: anEvent itemCategory) indexOf: anEvent itemClass name)
		column: 1
		parent: sysIndex.
	
	anEvent itemKind = #category ifTrue: [
		anEvent isAdded ifTrue: [
			self rowsInsertedFor: pkgIndex from: sysIndex row to: sysIndex row.
			self rowsChangedFor: nil from: pkgIndex row to: pkgIndex row].
		anEvent isRemoved ifTrue: [
			self rowsRemovedFor: pkgIndex from: sysIndex row to: sysIndex row.
			self rowsChangedFor: nil from: pkgIndex row to: pkgIndex row].
		anEvent isRenamed ifTrue: [
			sysIndex := self
				indexRow: (pkg systemCategories indexOf: anEvent newName)
				column: 1
				parent: pkgIndex.
			self rowsChangedFor: pkgIndex from: sysIndex row to: sysIndex row]].

	anEvent itemKind = #class ifTrue: [
		anEvent isAdded ifTrue: [
			self rowsInsertedFor: sysIndex from: classIndex row to: classIndex row.
			self rowsChangedFor: pkgIndex from: sysIndex row to: sysIndex row].
		anEvent isRenamed ifTrue: [
			self rowsChangedFor: sysIndex from: classIndex row to: classIndex row].
		anEvent isRemoved ifTrue: [
			self rowsRemovedFor: sysIndex from: classIndex row to: classIndex row.
			self rowsChangedFor: pkgIndex from: sysIndex row to: sysIndex row]
		].
	
	anEvent itemKind = #method ifTrue: [
		].! !

!UiSmalltalkModel methodsFor: 'model data' stamp: 'mt 10/14/2010 13:26:32.98'!
textFor: aModelIndex

	| col package systemCategory class protocol method |
	col := aModelIndex column.
	package := aModelIndex valueOfProperty: #package ifAbsent: [].
	systemCategory := aModelIndex valueOfProperty: #systemCategory  ifAbsent: [].
	class := aModelIndex valueOfProperty: #class ifAbsent: [].
	protocol := aModelIndex valueOfProperty: #protocol ifAbsent: [].
	method := aModelIndex valueOfProperty: #method ifAbsent: [].
	
	"Horizontal headers."
	aModelIndex row = 0 ifTrue: [
		col = 1 ifTrue: [^ '<b>Name</b>' asHtmlText].
		col = 2 ifTrue: [^ '<b>Size</b>' asHtmlText].
		col = 3 ifTrue: [^ '<b>Type</b>' asHtmlText]].
	
	"Vertical headers."
	aModelIndex column = 0 ifTrue: [
		^ aModelIndex row asString].
	
	package ifNotNil: [:p | 
		col = 1 ifTrue: [^ p name].
		col = 2 ifTrue: [^ ('<font color="#555555">', p systemCategories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">package</font>' asHtmlText]].
	
	systemCategory ifNotNil: [:cat | 
		| prefix |
		prefix := aModelIndex valueOfProperty: #systemCategoryPrefix.
		
		col = 2 ifTrue: [^ ('<font color="#555555">', (SystemOrganization listAtCategoryNamed: cat) size asString, '</font>') asHtmlText].
		
		cat = prefix ifFalse: [cat := cat allButFirst: prefix size + 1].
		col = 1 ifTrue: [^ cat].
		col = 3 ifTrue: [^ '<font color="#555555">system category</font>' asHtmlText]].
	
	class ifNotNil: [:c |
		col = 1 ifTrue: [^ c].
		col = 2 ifTrue: [^ ('<font color="#555555">', (Smalltalk at: c) organization categories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">class</font>' asHtmlText]].

	protocol ifNotNil: [:p |
		| organization |
		organization := aModelIndex valueOfProperty: #organization.
		col = 1 ifTrue: [^ p].
		col = 2 ifTrue: [^ ('<font color="#555555">', (organization listAtCategoryNamed: p) size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">protocol</font>' asHtmlText]].
	
	method ifNotNil: [:m |
		col = 1 ifTrue: [^ '#', m].
		col = 3 ifTrue: [^ '<font color="#555555">method</font>' asHtmlText]].
		
	^ nil! !

!UiViewColumnInfo methodsFor: 'private' stamp: 'mt 10/14/2010 13:31:56.097'!
formatHeaderItem

	| backgroundColor |
	backgroundColor := Color gray: 0.85.
	headerItem
		vResizing: #spaceFill;
		fillStyle: ((GradientFillStyle ramp: {
			0.0 -> Color white.
			0.1-> (backgroundColor adjustBrightness: 0.05).
			0.6 -> (backgroundColor darker)})
			origin: headerItem topLeft;
			direction: 0 @ headerItem height);
		borderStyle: (BorderStyle raised width: 1).! !
